@@ -9,7 +9,7 @@ import onnx
 from onnx import FunctionProto, GraphProto, ModelProto, TensorProto, ValueInfoProto
 
 import onnxscript.onnx_types
-import onnxscript.type_annotation
+from onnxscript._internal import type_annotation
 
 _SINGLE_INDENT = "    "
 
@@ -152,7 +152,10 @@ def _translate_value_infos(value_infos: Sequence[ValueInfoProto]) -> str:
 
 def _to_str(s):
     if isinstance(s, bytes):
-        return s.decode("utf-8")
+        try:
+            return s.decode("utf-8")
+        except UnicodeDecodeError:
+            pass
     return s
 
 
@@ -391,8 +394,8 @@ class _Exporter:
                 attributes.append((at.name, at.ref_attr_name))
                 continue
             value = _attribute_value(at)
-            if isinstance(value, str):
-                attributes.append((at.name, f"{value!r}"))
+            if isinstance(value, (str, bytes)):
+                attributes.append((at.name, repr(value)))
                 continue
             if isinstance(value, np.ndarray):
                 onnx_dtype = at.t.data_type
@@ -664,7 +667,7 @@ class _Exporter:
             self._names_used.add(attr_name)
             # A default type of INT is used for attribute parameters that are never used.
             type = type_map.get(attr_name, onnx.AttributeProto.INT)
-            typerep = onnxscript.type_annotation.onnx_attr_type_to_onnxscript_repr(type)
+            typerep = type_annotation.onnx_attr_type_to_onnxscript_repr(type)
             return f"{attr_name}: {typerep}"
 
         inputs = [self._translate_onnx_var(x) for x in funproto.input]
